//////////////////////////////////////////////////////////////////////////////
//
// Revision History
//
// v.1.0
// * GUI launcher.
// * Modified code to use TCHAR (for future Unicode support).
// * Refactored and cleaned up code.
// * Renamed to Blackhole Proxy (instead of BlackHoleProxy)
//
// v0.96
// * Bug fixes and code clean-up,
//  - Invalid small icon handle (causes access violation in BoundsChecker). (Thanks, Alec Han).
//  - Eliminate dependence in order-of-evaluation.
//  - Send HTTP header with ad replacement graphics.
// * Added version resource.
// * Debug output.
//
// v0.95
// * Added support for serving a PAC file.
// * Parameters are now case-insensitive.
// * Default to accept only local connection.
// * Single instance.
// * Added support to terminate the current instance.
// * Feedback message for http://no-ads.int/on and http://no-ads.int/off
// * Bug fixes and code clean-up.
//
// v0.90
// * Initial release.
//
// DISCLAIMER:
// The code is provided as-is.
// The author assumes no responsibility for any errors either
// in the comments or the code of this program.
// Bug fixes and suggestions are welcomed.
// Please e-mail larrywang10@hotmail.com.
// 
// Copyright 2003, Larry L. Wang. All Rights Reserved.
// Permission is given to modify and distribute this code,
// provided that the modifications are noted, and
// that this original copyright message is included.

#include "stdafx.h"
#include "BlackHoleProxy.h"
#include "ParseCmdLine.h"
#include "WindowsApp.h"
#include "ServerSock.h"

BOOL g_bDebugOutput = FALSE;

LPCSTR APP_NAME = "Blackhole Proxy";

int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
	INIT_ARGS args;
	args.bTerminate = FALSE;
	args.nPort = DEFAULT_PORT;
	args.nPacPort = -1; // Not used unless "-pac" is specified.
	args.szGraphicsFile[0] = _T('\0');
	args.bServer = FALSE;
#ifdef _DEBUG
	args.bDebugOutput = FALSE;
#endif

	ParseCmdLine(lpCmdLine, args);

	if (args.bTerminate)
	{
		if (TerminateWinAppInstances())
		{
			DisplayMessage(_T("Blackhole Proxy has been terminated."), -1, ERROR_LEVEL_INFO);
		}
		return TRUE;
	}

	return RunWindowsApp(hInstance, nCmdShow, args);
}

BOOL Initialize(const INIT_ARGS& args)
{
#ifdef _DEBUG
	g_bDebugOutput = args.bDebugOutput;
#endif
	g_bServer = args.bServer;

	if (!InitServerSock(args.nPort, args.nPacPort))
	{
		Uninitialize();
		return FALSE;
	}

	InitLocalHostAddress();

	if (_tcscmp(args.szGraphicsFile, _T("")) != 0)
	{
		SetGraphicsBuffer(args.szGraphicsFile);
	}

	SetHttpHeaderGraphics(args.szGraphicsFile);

	return TRUE;
}

void Uninitialize()
{
	UninitServerSock();
	delete[] g_pOptionalGraphics;
	g_pOptionalGraphics = NULL;
}

LPCTSTR GetFileExtension(LPCTSTR szFile)
{
	LPCTSTR szExt = NULL;
	// Determine the file type.
	int i = ReverseFind(szFile, '.');
	if (i >= 0)
	{
		szExt = szFile+i+1;
	}
	return szExt;
}

// Going from the end fo the beginning, find the first index containing the character.
int ReverseFind(LPCTSTR lpStr, TCHAR c)
{
	for (int i=strlen(lpStr)-1; i>=0; i--)
	{
		if (lpStr[i] == c)
		{
			return i;
		}
	}
	return -1;
}

void DisplayMessage(LPCTSTR szMessage /*=NULL*/, DWORD dwError /*=-1*/, ERROR_LEVEL errorLevel /*=ERROR_LEVEL_ERROR*/)
{
	if (g_bSilent)
	{
		return;
	}

	TCHAR szBuf[2048] = { _T('\0') };
	LPTSTR pBuf = szBuf;
	
	if (NULL != szMessage)
	{
		_tcscpy(szBuf, szMessage);
		pBuf = szBuf + _tcslen(szBuf);
	}

	if (dwError != -1)
	{
		// Append the system error message to the application error message.
		_tcscat(pBuf, _T("\r\n"));

		// Get the localized system message.
		FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, NULL, dwError, 0, pBuf + _tcslen("\r\n"), 256, NULL);
	}

	if (_T('\0') == szBuf[0])
	{
		_tcscpy(szBuf, _T("No error message available."));
	}

	WORD wType;
	switch (errorLevel)
	{
		case ERROR_LEVEL_INFO:
			wType = MB_ICONINFORMATION;
			break;
		case ERROR_LEVEL_WARNING:
			wType = MB_ICONEXCLAMATION;
			break;
		case ERROR_LEVEL_ERROR:
			wType = MB_ICONERROR;
			break;
	}
	::MessageBox(NULL, szBuf, APP_NAME, wType | MB_OK);
}

void DebugOutput(LPCTSTR szMsg)
{
#ifdef _DEBUG
	if (!g_bDebugOutput)
	{
		return;
	}

	HANDLE hFile;
	if ((hFile=CreateFile("BlackHoleProxy.log", GENERIC_WRITE, 0, NULL, OPEN_ALWAYS, 0, NULL)) != INVALID_HANDLE_VALUE)
	{
		DWORD dwBytesWritten;
		SetFilePointer(hFile, 0, NULL, FILE_END);
		WriteFile(hFile, szMsg, strlen(szMsg), &dwBytesWritten, NULL);
		CloseHandle(hFile);
	}
#endif
}
