// ParseCmdLine.cpp

#include "stdafx.h"
#include "BlackHoleProxy.h"
#include "ParseCmdLine.h"
#include "ServerSock.h"

BOOL g_bSilent = FALSE;

// Helper functions
int ParsePortParam(LPCTSTR lpToken);
int ParsePacParam(LPCTSTR lpToken);
LPCTSTR ParseGraphicsParam(LPTSTR lpToken);

void SetGraphicsFileName(LPCTSTR szFile, LPTSTR szGraphicsFile);

void ParseCmdLine(LPTSTR lpCmdLine, INIT_ARGS& args)
{
	// Convert the command line to lower case (to make it case-insensitive).
	for (unsigned int i=0; i<_tcslen(lpCmdLine); i++)
	{
		lpCmdLine[i] = tolower(lpCmdLine[i]);
	}

	// Parse the command line.
	LPTSTR lpToken = _tcstok(lpCmdLine, SEPARATOR);
	while (lpToken != NULL)
	{
		if (_tcsncmp(lpToken, TERMINATE_PARAM, _tcslen(TERMINATE_PARAM)) == 0)
		{
			args.bTerminate = TRUE;
		}
#ifdef _DEBUG
		else if (_tcsncmp(lpToken, LOG_PARAM, _tcslen(LOG_PARAM)) == 0)
		{
			args.bDebugOutput = TRUE;
		}
#endif
		else if (_tcsncmp(lpToken, SERVER_PARAM, _tcslen(SERVER_PARAM)) == 0)
		{
			args.bServer = TRUE;
		}
		else if (_tcsncmp(lpToken, PORT_PARAM, _tcslen(PORT_PARAM)) == 0)
		{
			args.nPort = ParsePortParam(lpToken);
		}
		else if (_tcsncmp(lpToken, PAC_PARAM, _tcslen(PAC_PARAM)) == 0)
		{
			g_bPac = TRUE;
			args.nPacPort = ParsePacParam(lpToken);
		}
		else if (_tcsncmp(lpToken, GRAPHICS_PARAM, _tcslen(GRAPHICS_PARAM)) == 0)
		{
			SetGraphicsFileName(
				ParseGraphicsParam(lpToken),
				args.szGraphicsFile);
		}
		else if (_tcsncmp(lpToken, SILENT_PARAM, _tcslen(SILENT_PARAM)) == 0)
		{
			g_bSilent = TRUE;
		}
		lpToken = strtok(NULL, SEPARATOR);
	}
}

LPCTSTR ParseGraphicsParam(LPTSTR lpToken)
{
	LPTSTR szFile = lpToken + _tcslen(GRAPHICS_PARAM) + 1; // skip the space after the params
			 
	// Remove trailing spaces.
	for (int i=_tcslen(szFile)-1; i>0; i--)
	{
		if (!_istspace(szFile[i]))
		{
			break;
		}
		szFile[i] = _T('\0');
	}
	return szFile;
}

int ParsePortParam(LPCTSTR lpToken)
{
	int nPort = DEFAULT_PORT;
	if (_tcslen(lpToken) > _tcslen(PORT_PARAM) + 1) // (account for the space)
	{
		nPort = _ttoi(lpToken + _tcslen(PORT_PARAM) + 1); // skip the space after the param.
		if (0 == nPort)
		{
			TCHAR szBuf[512];
			_stprintf(szBuf, _T("The value for the \"-%s\" parameter must be a number.\nUsing default port %d."), PORT_PARAM, DEFAULT_PORT);
			DisplayMessage(szBuf, -1, ERROR_LEVEL_WARNING);
			nPort = DEFAULT_PORT;
		}
	}
	return nPort;
}

int ParsePacParam(LPCTSTR lpToken)
{
	int nPacPort = DEFAULT_PAC_PORT;
	if (_tcslen(lpToken) > _tcslen(PAC_PARAM) + 1) // (account for the space)
	{
		LPCTSTR lpszPacPort = lpToken + _tcslen(PAC_PARAM) + 1;
		if (lpszPacPort[0] != _T('\0'))
		{
			nPacPort = _ttoi(lpszPacPort); 
			if (0 == nPacPort)
			{
				TCHAR szBuf[512];
				_stprintf(szBuf, _T("\"%s\" is not a valid value for the \"-%s\" parameter. Using default port %d."), lpszPacPort, PAC_PARAM, DEFAULT_PAC_PORT);
				DisplayMessage(szBuf, -1, ERROR_LEVEL_WARNING);
				nPacPort = DEFAULT_PAC_PORT;
			}
		}
	}
	return nPacPort;
}

void SetGraphicsFileName(LPCTSTR szFile, LPTSTR szGraphicsFile)
{
	// GetFileAttributes is a quick-and-dirty way to determine if a file exists.
	if (INVALID_FILE_ATTRIBUTES == GetFileAttributes(szFile))
	{
		// File not opened
		TCHAR szBuf[512];
		_stprintf(szBuf, _T("Unable to open %s. Using default graphics."), szFile);
		DisplayMessage(szBuf, -1, ERROR_LEVEL_WARNING);
	}
	else
	{
		_tcscpy(szGraphicsFile, szFile);
	}
}
