// ServerSock.cpp

#include "stdafx.h"
#include "BlackHoleProxy.h"
#include "ServerSock.h"
#include "WindowsApp.h"

#define BUF_SIZE 1024

HANDLE g_hSockThread = NULL;
HANDLE g_hPacSockThread = NULL;
DWORD g_dwPacThreadID = 0;
SOCKET g_hServerSock = INVALID_SOCKET;
SOCKET g_hPacServerSock = INVALID_SOCKET;
BOOL g_bPac = FALSE;
BOOL g_bServer = FALSE;

// HTTP headers
static LPCSTR NO_ADS_ON_STRING = "Content-length: 17\r\n\r\nNo-ads is now ON.";
static LPCSTR NO_ADS_OFF_STRING = "Content-length: 18\r\n\r\nNo-ads is now OFF.";
static LPCSTR EMPTY_STRING = "Content-length: 0\r\n\r\n";
static LPCSTR HTTP_HEADER_GRAPHICS = "HTTP/1.0 200 OK\r\nContent-type: image/%s\r\nContent-length: %d\r\n\r\n";
static LPCSTR HTTP_HEADER_TEXT = "HTTP/1.0 200 OK\r\nContent-Type: text/html\r\n";
static LPCSTR HTTP_HEADER_PAC = "HTTP/1.0 200 OK\r\nContent-type: application/x-ns-proxy-autoconfig\r\nContent-length: %d\r\n\r\n";
static IN_ADDR LOCAL_HOST_ADDR;

// Default graphics for blocked sites,
// courtesy John LoVerso (http://www.schooner.com/~loverso/no-ads/noproxy.clear.gif)
const BYTE DEFAULT_GRAPHICS[] = {
0x47,0x49,0x46,0x38,0x39,0x61,0x01,0x00,0x01,0x00,0xf0,0x00,0x00,0xff,0xff,0xff,0x00,0x00,0x00,0x21,0xf9,
0x04,0x01,0x00,0x00,0x00,0x00,0x2c,0x00,0x00,0x00,0x00,0x01,0x00,0x01,0x00,0x00,0x02,0x02,0x44,0x01,0x00,
0x3b
};

LPBYTE g_pOptionalGraphics = NULL; // buffer to store the optional default graphics
const BYTE* g_pGraphics = DEFAULT_GRAPHICS;
UINT g_dwGraphicsSize = sizeof(DEFAULT_GRAPHICS); // size of the graphics buffer
char g_szHttpHeaderGraphics[1024] = { _T('\0') }; // HTTP header for the graphics.

HANDLE CreateSockThread(SOCKET*, int);
DWORD WINAPI SockThreadProc(LPVOID);
void ProcessClientSock(SOCKET, BOOL);
void SendData(SOCKET, LPCSTR, LPCVOID, DWORD);
void SendPacFile(SOCKET, LPTSTR);

BOOL IsLocalHost(SOCKADDR_IN*);

BOOL IsPacThread();

HANDLE CreateSockThread(SOCKET*, int);

void InitLocalHostAddress()
{
	// Initialize global IN_ADDR for localhost address.
	LOCAL_HOST_ADDR.S_un.S_addr = inet_addr("127.0.0.1");
}

BOOL InitServerSock(int nPort, int nPacPort)
{
	WSADATA WSAData;                    // Contains details of the Winsock
										// implementation
	// Initialize Winsock.
	if (WSAStartup(MAKEWORD(1,1), &WSAData) != 0) 
	{
		DisplayMessage(_T("Unable to initialize Winsock."), GetLastError());
		return FALSE;
	}

	if ((g_hSockThread=CreateSockThread(&g_hServerSock, nPort)) == NULL)
	{
		//DisplayMessage(_T("Unable to create proxy thread."));
		return FALSE;
	}

	// Create a socket thread for PAC, if necessary
	if (g_bPac && nPacPort != nPort)
	{
		if ((g_hPacSockThread=CreateSockThread(&g_hPacServerSock, nPacPort)) == NULL)
		{
			//DisplayMessage(_T("Unable to create PAC thread."));
			return FALSE;
		}
	}

	return TRUE;
}

HANDLE CreateSockThread(SOCKET* pSocket, int port)
{
	SOCKADDR_IN sock_addr; // Local socket address

	if (INVALID_SOCKET == (*pSocket=socket(AF_INET, SOCK_STREAM, 0)))
	{
		DisplayMessage(_T("Unable to create a socket."), GetLastError());
		return NULL;
	}

	// Fill out the local socket's address information.
	sock_addr.sin_family = AF_INET;
	sock_addr.sin_port = htons(port);
	sock_addr.sin_addr.s_addr = htonl(INADDR_ANY);

	// Associate the local address with socket.
	if (SOCKET_ERROR == bind(*pSocket, (struct sockaddr *)&sock_addr, sizeof(sock_addr))) 
	{
		char szBuf[512];
		sprintf(szBuf, "Unable to bind port %d.", port);
		DisplayMessage(szBuf, GetLastError());
		closesocket(*pSocket);
		return NULL;
	}

	// Successfully created socket. Launch a thread to listen for connections from clients.
	DWORD dwThreadID;
	return CreateThread(NULL, 0, SockThreadProc, (LPVOID)*pSocket, 0, (pSocket == &g_hPacServerSock)? &g_dwPacThreadID: &dwThreadID); // save the thread ID for the PAC thread (if applicable).
}

void UninitServerSock()
{
	// Stop listening for connections from clients.
	closesocket(g_hServerSock);
	closesocket(g_hPacServerSock);
	WSACleanup();
	CloseHandle(g_hSockThread);
	CloseHandle(g_hPacSockThread);
}

DWORD WINAPI SockThreadProc(LPVOID lpVoid)
{
	SOCKADDR_IN sock_addr; // Receives the address of the connecting entity.
	int sockaddr_len = sizeof(sock_addr);
	SOCKET hServerSock = (SOCKET)lpVoid;
	BOOL bCheckPac = g_bPac && IsPacThread(); // TRUE if this thread should check for a PAC request.
	
	while (TRUE)
	{
		// Establish a socket to listen for incoming connections.
		if (SOCKET_ERROR == listen(hServerSock, 1))
		{
			DisplayMessage(_T("Unable to listen to socket."), GetLastError());
			SendMessage(g_hWnd, WM_CLOSE, 0, 0);
			return FALSE;
		}

		// Accept an incoming connection attempt on hServerSock.
		SOCKET hClientSock = accept(hServerSock, 
			               (struct sockaddr *)&sock_addr, 
				           (int *)&sockaddr_len);

		if (INVALID_SOCKET == hClientSock) 
		{
			// Probably caused by the WM_DESTROY message, which closed hServerSock
			DebugOutput(_T("Listening socket was closed.\r\n"));
			return TRUE;
		}

		// order-of-evaluation is left-to-right
		// Process only local packets unless the "-server" flag was specified.
		if (g_bServer || IsLocalHost(&sock_addr))
		{
			ProcessClientSock(hClientSock, bCheckPac);
		}
	}

	DebugOutput(_T("Thread signalled to close.\r\n"));
	return TRUE;
}

// The same function below processes the requests
// for regular HTTP data (ads) and PAC file,
// even though the requests come through different ports.
void ProcessClientSock(SOCKET hClientSock, BOOL bCheckPac)
{
	int len = 0;
	do
	{
		char szRequest[BUF_SIZE];

		// Receive data from the client.
		len = recv(hClientSock, szRequest, BUF_SIZE-1, 0);

		// Check if there is any data received.
		if (SOCKET_ERROR == len || 0 == len)
		{
			break;
		}
		szRequest[len] = 0;

		DebugOutput(szRequest);

		// Parse the data for GET command.
		LPSTR lpToken = strtok(szRequest, " ");
		while (lpToken != NULL)
		{
			// Find the token after the GET command,
			// and see if the requested object ends with
			// .txt or .js.
			// If so, return empty text instead of a replacement graphics.
			// This eliminates a parse error from a web page
			// that expects text (like when it includes a JavaScript file).
			if (strcmp(lpToken, "GET") == 0)
			{
				if ((lpToken=strtok(NULL, " ")) != NULL)
				{
					if (strcmp(lpToken, "http://no-ads.int/on") == 0)
					{
						SendData(hClientSock, HTTP_HEADER_TEXT, NO_ADS_ON_STRING, strlen(NO_ADS_ON_STRING));
						break;
					}
					
					if (strcmp(lpToken, "http://no-ads.int/off") == 0)
					{
						SendData(hClientSock, HTTP_HEADER_TEXT, NO_ADS_OFF_STRING, strlen(NO_ADS_OFF_STRING));
						break;
					}

					// Look for .pac, .txt, or .js
					int i = ReverseFind(lpToken, '.');
					if (i >= 0)
					{
						if (bCheckPac)
						{
							// Look for .pac
							// lpToken+i+1 points to the character after the period.
							if (strcmp(lpToken+i+1, "pac") == 0)
							{
								SendPacFile(hClientSock, lpToken);
								break;
							}
						}
						
						// Look for .txt, and .js
						if (strcmp(lpToken+i+1, "txt") == 0 ||
							strcmp(lpToken+i+1, "js") == 0 ||
							strcmp(lpToken+i+1, "php") == 0 ||
							strncmp(lpToken+i+1, "php?", 4) == 0)
						{
							// Send empty text to the client.
							SendData(hClientSock, HTTP_HEADER_TEXT, EMPTY_STRING, strlen(EMPTY_STRING));
							DebugOutput(HTTP_HEADER_TEXT);
							break;
						}
					}

					// No special URL found.
					// Send replacement graphics to the client.
					SendData(hClientSock, g_szHttpHeaderGraphics, g_pGraphics, g_dwGraphicsSize);
					DebugOutput(g_szHttpHeaderGraphics);
				}
				break;
			}
			lpToken = strtok(NULL, " ");
		}
	} while (BUF_SIZE == len);

	// Disable both sending and receiving on hClientSock.
	shutdown(hClientSock, 0x02);
	closesocket(hClientSock);
}

void SendData(SOCKET hSock, LPCSTR szHttpHeader, LPCVOID szData, DWORD dwDataSize)
{
	send(hSock, szHttpHeader, strlen(szHttpHeader), 0);
	send(hSock, (LPCSTR)szData, dwDataSize, 0);
}

void SendPacFile(SOCKET hClientSock, LPTSTR szFileName)
{
	// Return PAC file with MIME type "application/x-ns-proxy-autoconfig"
	// (see content of HTTP_HEADER_PAC).
	// (Required for some browsers (Netscape, Opera?) to work).
	HANDLE hFile;
	if ((hFile=CreateFile(szFileName, GENERIC_READ, 0, NULL, OPEN_EXISTING, 0, NULL)) != INVALID_HANDLE_VALUE)
	{
		// Send the HTTP header.
		DWORD dwSize = GetFileSize(hFile, NULL);
		char szBuf[4096];
		sprintf(szBuf, HTTP_HEADER_PAC, dwSize);
		send(hClientSock, szBuf, strlen(szBuf), 0);

		// Send the PAC file data in 4K chunks.
		DWORD dwBytesRead;
		while (ReadFile(hFile, szBuf, 4096, &dwBytesRead, NULL))
		{
			if (dwBytesRead <= 0) // dwBytesRead == 0 when we reach EOF.
			{
				break;
			}
			send(hClientSock, szBuf, dwBytesRead, 0);
		}
		CloseHandle(hFile);
	}
}

// returns TRUE is sock_addr contains a local host address (127.0.0.1)
BOOL IsLocalHost(SOCKADDR_IN* sock_addr)
{
	return memcmp(&sock_addr->sin_addr.S_un.S_un_b, &LOCAL_HOST_ADDR, sizeof(LOCAL_HOST_ADDR)) == 0;
}

BOOL IsPacThread()
{
	return NULL == g_hPacSockThread ||				// same thread handles PAC and HTML requests.
		   GetCurrentThreadId() == g_dwPacThreadID; // PAC thread.
}

void SetHttpHeaderGraphics(LPCTSTR szGraphicsFile)
{
	sprintf(g_szHttpHeaderGraphics, 
			HTTP_HEADER_GRAPHICS, 
			(_tcscmp(szGraphicsFile, _T("")) != 0) ? 
				GetFileExtension(szGraphicsFile) : 
				"gif", // default to image/gif (for the HTTP header).
			g_dwGraphicsSize);
}

BOOL SetGraphicsBuffer(LPCTSTR szGraphicsFile)
{
	// Read the graphics file supplied in the command line.
	HANDLE hFile;
	if ((hFile=CreateFile(szGraphicsFile, GENERIC_READ, 0, NULL, OPEN_EXISTING, 0, NULL)) == INVALID_HANDLE_VALUE)
	{
		// File not opened
		TCHAR szBuf[512];
		_stprintf(szBuf, _T("Unable to open %s. Using default graphics."), szGraphicsFile);
		DisplayMessage(szBuf, -1, ERROR_LEVEL_WARNING);
		return FALSE;
	}
	else
	{
		// Assume that the graphics file size is small.
		g_dwGraphicsSize = GetFileSize(hFile, NULL);
		g_pOptionalGraphics = new BYTE[g_dwGraphicsSize];

		DWORD dwBytesRead;
		if (ReadFile(hFile, g_pOptionalGraphics, g_dwGraphicsSize, &dwBytesRead, NULL))
		{
			g_pGraphics = g_pOptionalGraphics;
		}
		else
		{
			// Use DEFAULT_GRAPHICS
			delete[] g_pOptionalGraphics;
			g_dwGraphicsSize = sizeof(DEFAULT_GRAPHICS);
			g_pOptionalGraphics = NULL;
		} 

		CloseHandle(hFile);
		return TRUE;
	}
}
