// WindowsApp.cpp

#include "stdafx.h"
#include "BlackHoleProxy.h"
#include "WindowsApp.h"

HWND g_hWnd = NULL;

BOOL InitApplication(HINSTANCE); 
BOOL InitInstance(HINSTANCE, int); 
LRESULT CALLBACK MainWndProc(HWND, UINT, WPARAM, LPARAM);

BOOL RunWindowsApp(HINSTANCE hInstance, int nCmdShow, const INIT_ARGS& args)
{
	if (args.bTerminate)
	{
		if (TerminateWinAppInstances())
		{
			DisplayMessage(_T("Blackhole Proxy has been terminated."), -1, ERROR_LEVEL_INFO);
		}
		return TRUE;
	}
/*
	if (FindWindow(WIN_CLASS_NAME, NULL) != NULL)
	{
		g_bDebugOutput = TRUE;
		DebugOutput("Another instance is running. Exiting.\n");
		return TRUE;
	}
*/
	if (!InitApplication(hInstance))
	{
		return FALSE; 
	}

	// Put InitInstance after parsing the parameter to
	// avoid closing ourselves when "-singleinst" is specified.
	if (!InitInstance(hInstance, nCmdShow))
	{
		return FALSE; 
	}

	if (!Initialize(args))
	{
		return FALSE;
	}

	DisplayMessage(_T("Blackhole Proxy is now running."), -1, ERROR_LEVEL_INFO);

	// Main message loop:
	MSG msg;
	BOOL bRet;
	for (;;)
	{
		bRet = GetMessage(&msg, (HWND)NULL, 0, 0);
		if (0 == bRet || -1 == bRet)
		{
			break;
		}
		TranslateMessage(&msg); 
		DispatchMessage(&msg); 
	}

	return msg.wParam; 
}

BOOL TerminateWinAppInstances()
{
	// Close all other instances.
	HWND hWnd;
	BOOL res = FALSE;
	while ((hWnd=FindWindow(WIN_CLASS_NAME, NULL)) != NULL)
	{
		if (!res)
		{
			res = TRUE;
		}
		PostMessage(hWnd, WM_CLOSE, 0, 0);
	}

	return res;
}

BOOL InitApplication(HINSTANCE hInstance) 
{ 
	WNDCLASSEX wcx; 
 
	// Fill in the window class structure with parameters 
	// that describe the main window. 
 
	wcx.cbSize = sizeof(wcx);          // size of structure 
	wcx.style = CS_HREDRAW | CS_VREDRAW; // redraw if size changes 
	wcx.lpfnWndProc = MainWndProc;     // points to window procedure 
	wcx.cbClsExtra = 0;                // no extra class memory 
	wcx.cbWndExtra = 0;                // no extra window memory 
	wcx.hInstance = hInstance;         // handle to instance 
	wcx.hIcon = NULL;				   // predefined app. icon 
	wcx.hCursor = LoadCursor(NULL, IDC_ARROW); // predefined arrow 
	wcx.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH); // white background brush 
	wcx.lpszMenuName = NULL;
	wcx.lpszClassName = WIN_CLASS_NAME;  // name of window class 
	wcx.hIconSm = NULL;
 
    BOOL bRet = RegisterClassEx(&wcx);
	if (!bRet)
	{
		DisplayMessage(_T("Unable to register window class. The application will terminate."));
	}
	return bRet;
} 
 
BOOL InitInstance(HINSTANCE hInstance, int nCmdShow) 
{ 
	// Create the main window. 
	g_hWnd = CreateWindow( 
		WIN_CLASS_NAME,      // name of window class 
		"",					 // title-bar string 
		WS_OVERLAPPEDWINDOW, // top-level window 
		CW_USEDEFAULT,       // default horizontal position 
		CW_USEDEFAULT,       // default vertical position 
		250,				 // window width 
		250,				 // window height 
		(HWND)NULL,          // no owner window 
		(HMENU)NULL,         // use class menu 
		hInstance,           // handle to application instance 
		(LPVOID) NULL);      // no window-creation data 
 
	if (!g_hWnd) 
	{
		DisplayMessage(_T("Unable to create window. The application will terminate."));
		return FALSE; 
	}
 
	// Show the window and send a WM_PAINT message to the window 
	// procedure.
#ifdef _DEBUG
    ShowWindow(g_hWnd, nCmdShow);
#else
	ShowWindow(g_hWnd, SW_HIDE);
#endif
	UpdateWindow(g_hWnd); 
    return TRUE; 
}

LRESULT CALLBACK MainWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message) 
	{
		case WM_DESTROY:
			Uninitialize();
			PostQuitMessage(0);
			break;
		default:
			return DefWindowProc(hWnd, message, wParam, lParam);
	}
	return 0;
}

